@@ -10,6 +10,11 @@ logging.basicConfig(level=level)
 logger = logging.getLogger(__name__)
 
 
+def set_api_key(key) -> None:
+    '''set OpenAI API key'''
+    openai.api_key = key
+
+
 class GPT3Generator:
     def __init__(self,
                  input_text: str = 'Input',
@@ -21,10 +26,7 @@ class GPT3Generator:
         self.output_text = output_text
         self.instructions: str = ''
         self.examples: Dict[str, str] = {}
-
-    def set_key(self, key: str) -> None:
-        '''set OpenAI API key'''
-        openai.api_key = key
+        self.full_prompt: str = ''
 
     def set_instructions(self, instructions: str) -> None:
         '''Set instructions for language generation (followed by examples)
@@ -47,7 +49,7 @@ class GPT3Generator:
         output (str): output text in example
         '''
         if input in self.examples:
-            logger.warning(f'Example already exists. This will cause duplicate examples in the prompt.')
+            logger.warning(f'Example already exists. This will create duplicate examples in the prompt.')
         else:
             self.examples[input] = output
 
@@ -60,25 +62,25 @@ class GPT3Generator:
 
     def get_prompt(self) -> str:
         '''Returns the prompt used for language generation'''
-        if self.instructions == '' and not self.examples:
-            raise ValueError('No prompt has been provided. Please use at least one of set_instructions(), add_example().')
-
         expanded_examples = '\n\n'.join([f'{self.input_text}: {k}\n{self.output_text}: {v}' for k, v in self.examples.items()])
-        
-        if self.examples:
-            expanded_examples += f'\n\n{self.input_text}:'
-
         return f'{self.instructions}{expanded_examples}'
 
     def get_gpt3_response(self, starting_text: str, **kwargs) -> openai.openai_response:
         '''Call OpenAI API to get the prompt'''
-        prompt = self.get_prompt() + ' ' + starting_text
-        try:
-            return openai.Completion.create(prompt=prompt, **kwargs)
-        except openai.error.AuthenticationError:
-            raise Exception('Unable to authenticate: use set_key() to set OpenAI key.')
+        if self.examples:
+            starting_text = f'\n\n{self.input_text}: {starting_text}\n'
+        self.full_prompt = self.get_prompt() + starting_text
 
-    def generate(self, starting_text: str, **kwargs) -> None:
+        try:
+            return openai.Completion.create(prompt=self.full_prompt, **kwargs)
+        except openai.error.AuthenticationError:
+            raise Exception('Unable to authenticate: use set_api_key() to set OpenAI key.')
+
+    def generate(self, prompt: str, **kwargs) -> Dict[str, str]:
         '''Get generated text'''
-        gpt3_response = self.get_gpt3_response(starting_text, **kwargs)
-        return gpt3_response['choices'][0]['text'].strip()
+        gpt3_response = self.get_gpt3_response(prompt, **kwargs)
+        generated_text = gpt3_response['choices'][0]['text'].strip()
+        return {
+            'generated_text': generated_text,
+            'full_prompt': self.full_prompt
+        }
